from pathlib import Path

import dmgbuild


def test_sample_settings(tmp_path):
    """Test with the sample settings.py file."""
    target = tmp_path / "out.dmg"
    dmgbuild.build_dmg(
        str(target), "Test", str(Path(__file__).parent / "examples" / "settings.py")
    )

    assert target.exists()


def test_sample_json(tmp_path):
    """Test with the sample settings.json file."""
    target = tmp_path / "out.dmg"
    dmgbuild.build_dmg(
        str(target), "Test", str(Path(__file__).parent / "examples" / "settings.json")
    )

    assert target.exists()



import os.path
import plistlib

#
# Example settings file for dmgbuild
#

# Use like this: dmgbuild -s settings.py "Test Volume" test.dmg

# You can actually use this file for your own application (not just TextEdit)
# by doing e.g.
#
#   dmgbuild -s settings.py -D app=/path/to/My.app "My Application" MyApp.dmg

# .. Useful stuff ..............................................................

application = defines.get("app", "/System/Applications/TextEdit.app")  # noqa: F821
appname = os.path.basename(application)


def icon_from_app(app_path):
    plist_path = os.path.join(app_path, "Contents", "Info.plist")
    with open(plist_path, "rb") as f:
        plist = plistlib.load(f)
    icon_name = plist["CFBundleIconFile"]
    icon_root, icon_ext = os.path.splitext(icon_name)
    if not icon_ext:
        icon_ext = ".icns"
    icon_name = icon_root + icon_ext
    return os.path.join(app_path, "Contents", "Resources", icon_name)


# .. Basics ....................................................................

# Uncomment to override the output filename
# filename = 'test.dmg'

# Uncomment to override the output volume name
# volume_name = 'Test'

# Volume format (see hdiutil create -help)
format = defines.get("format", "UDBZ")  # noqa: F821

# Compression level (if relevant)
# compression_level = 9

# Volume size
size = defines.get("size", None)  # noqa: F821

# Files to include
files = [application]

# Symlinks to create
symlinks = {"Applications": "/Applications"}

# Files to hide
# hide = [ 'Secret.data' ]

# Files to hide the extension of
# hide_extension = [ 'README.rst' ]

# Volume icon
#
# You can either define icon, in which case that icon file will be copied to the
# image, *or* you can define badge_icon, in which case the icon file you specify
# will be used to badge the system's Removable Disk icon. Badge icons require
# pyobjc-framework-Quartz.
#
# icon = '/path/to/icon.icns'
badge_icon = icon_from_app(application)

# Where to put the icons
icon_locations = {appname: (140, 120), "Applications": (500, 120)}

# .. Window configuration ......................................................

# Background
#
# This is a STRING containing any of the following:
#
#    #3344ff          - web-style RGB color
#    #34f             - web-style RGB color, short form (#34f == #3344ff)
#    rgb(1,0,0)       - RGB color, each value is between 0 and 1
#    hsl(120,1,.5)    - HSL (hue saturation lightness) color
#    hwb(300,0,0)     - HWB (hue whiteness blackness) color
#    cmyk(0,1,0,0)    - CMYK color
#    goldenrod        - X11/SVG named color
#    builtin-arrow    - A simple built-in background with a blue arrow
#    /foo/bar/baz.png - The path to an image file
#
# The hue component in hsl() and hwb() may include a unit; it defaults to
# degrees ('deg'), but also supports radians ('rad') and gradians ('grad'
# or 'gon').
#
# Other color components may be expressed either in the range 0 to 1, or
# as percentages (e.g. 60% is equivalent to 0.6).
background = "builtin-arrow"

show_status_bar = False
show_tab_view = False
show_toolbar = False
show_pathbar = False
show_sidebar = False
sidebar_width = 180

# Window position in ((x, y), (w, h)) format
window_rect = ((100, 100), (640, 280))

# Select the default view; must be one of
#
#    'icon-view'
#    'list-view'
#    'column-view'
#    'coverflow'
#
default_view = "icon-view"

# General view configuration
show_icon_preview = False

# Set these to True to force inclusion of icon/list view settings (otherwise
# we only include settings for the default view)
include_icon_view_settings = "auto"
include_list_view_settings = "auto"

# .. Icon view configuration ...................................................

arrange_by = None
grid_offset = (0, 0)
grid_spacing = 100
scroll_position = (0, 0)
label_pos = "bottom"  # or 'right'
text_size = 16
icon_size = 128

# .. List view configuration ...................................................

# Column names are as follows:
#
#   name
#   date-modified
#   date-created
#   date-added
#   date-last-opened
#   size
#   kind
#   label
#   version
#   comments
#
list_icon_size = 16
list_text_size = 12
list_scroll_position = (0, 0)
list_sort_by = "name"
list_use_relative_dates = True
list_calculate_all_sizes = (False,)
list_columns = ("name", "date-modified", "size", "kind", "date-added")
list_column_widths = {
    "name": 300,
    "date-modified": 181,
    "date-created": 181,
    "date-added": 181,
    "date-last-opened": 181,
    "size": 97,
    "kind": 115,
    "label": 100,
    "version": 75,
    "comments": 300,
}
list_column_sort_directions = {
    "name": "ascending",
    "date-modified": "descending",
    "date-created": "descending",
    "date-added": "descending",
    "date-last-opened": "descending",
    "size": "descending",
    "kind": "ascending",
    "label": "ascending",
    "version": "ascending",
    "comments": "ascending",
}

# .. License configuration .....................................................

# Text in the license configuration is stored in the resources, which means
# it gets stored in a legacy Mac encoding according to the language.  dmgbuild
# will *try* to convert Unicode strings to the appropriate encoding, *but*
# you should be aware that Python doesn't support all of the necessary encodings;
# in many cases you will need to encode the text yourself and use byte strings
# instead here.

# Recognized language names are:
#
#  af_ZA, ar, be_BY, bg_BG, bn, bo, br, ca_ES, cs_CZ, cy, da_DK, de_AT, de_CH,
#  de_DE, dz_BT, el_CY, el_GR, en_AU, en_CA, en_GB, en_IE, en_SG, en_US, eo,
#  es_419, es_ES, et_EE, fa_IR, fi_FI, fo_FO, fr_001, fr_BE, fr_CA, fr_CH,
#  fr_FR, ga-Latg_IE, ga_IE, gd, grc, gu_IN, gv, he_IL, hi_IN, hr_HR, hu_HU,
#  hy_AM, is_IS, it_CH, it_IT, iu_CA, ja_JP, ka_GE, kl, ko_KR, lt_LT, lv_LV,
#  mk_MK, mr_IN, mt_MT, nb_NO, ne_NP, nl_BE, nl_NL, nn_NO, pa, pl_PL, pt_BR,
#  pt_PT, ro_RO, ru_RU, se, sk_SK, sl_SI, sr_RS, sv_SE, th_TH, to_TO, tr_TR,
#  uk_UA, ur_IN, ur_PK, uz_UZ, vi_VN, zh_CN, zh_TW

license = {
    "default-language": "en_US",
    "licenses": {
        # For each language, the text of the license.  This can be plain text,
        # RTF (in which case it must start "{\rtf1"), or a path to a file
        # containing the license text.  If you're using RTF,
        # watch out for Python escaping (or read it from a file).
        "en_GB": b"""{\\rtf1\\ansi\\ansicpg1252\\cocoartf1504\\cocoasubrtf820
 {\\fonttbl\\f0\\fnil\\fcharset0 Helvetica-Bold;\\f1\\fnil\\fcharset0 Helvetica;}
 {\\colortbl;\\red255\\green255\\blue255;\\red0\\green0\\blue0;}
 {\\*\\expandedcolortbl;;\\cssrgb\\c0\\c0\\c0;}
 \\paperw11905\\paperh16837\\margl1133\\margr1133\\margb1133\\margt1133
 \\deftab720
 \\pard\\pardeftab720\\sa160\\partightenfactor0

 \\f0\\b\\fs60 \\cf2 \\expnd0\\expndtw0\\kerning0
 \\up0 \\nosupersub \\ulnone \\outl0\\strokewidth0 \\strokec2 Test License\\
 \\pard\\pardeftab720\\sa160\\partightenfactor0

 \\fs36 \\cf2 \\strokec2 What is this?\\
 \\pard\\pardeftab720\\sa160\\partightenfactor0

 \\f1\\b0\\fs22 \\cf2 \\strokec2 This is the English license. It says what you are allowed to do with this software.\\
 \\
 }""",
        "de_DE": "Ich bin ein Berliner. Bielefeld gibt's doch gar nicht.",
    },
    "buttons": {
        # For each language, text for the buttons on the licensing window.
        #
        # Default buttons and text are built-in for the following languages:
        #
        #   da_DK: Danish
        #   de_DE: German
        #   en_AU: English (Australian)
        #   en_GB: English (UK)
        #   en_NZ: English (New Zealand)
        #   en_US: English (US)
        #   es_ES: Spanish
        #   fr_CA: French (Canadian)
        #   fr_FR: French
        #   it_IT: Italian
        #   ja_JP: Japanese
        #   nb_NO: Norsk
        #   nl_BE: Flemish
        #   nl_NL: Dutch
        #   pt_BR: Brazilian Portuguese
        #   pt_PT: Portugese
        #   sv_SE: Swedish
        #   zh_CN: Simplified Chinese
        #   zh_TW: Traditional Chinese
        #
        # You don't need to specify them for those languages; if you fail to
        # specify them for some other language, English will be used instead.
        "en_US": (
            b"English",
            b"Agree!",
            b"Disagree!",
            b"Print!",
            b"Save!",
            b'Do you agree or not? Press "Agree" or "Disagree".',
        ),
    },
}



import os

language_names_map = {
    "da_DK": "Dansk",
    "de_DE": "Deutsch",
    "en_AU": "English",
    "en_GB": "English",
    "en_NZ": "English",
    "en_US": "English",
    "es_ES": "Español",
    "fr_CA": "Français",
    "fr_FR": "Français",
    "it_IT": "Italiano",
    "ja_JP": "Japanese",
    "nb_NO": "Norsk",
    "nl_BE": "Nederlands",
    "nl_NL": "Nederlands",
    "pt_BR": "Português",
    "pt_PT": "Português",
    "sv_SE": "Svensk",
    "zh_CN": "Simplified Chinese",
    "zh_TW": "Traditional Chinese",
}

# Buttons (these come from the SLAResources file which you can find in the SLA
#          SDK on developer.apple.com)
default_buttons = {
    "English": (
        b"English",
        b"Agree",
        b"Disagree",
        b"Print",
        b"Save",
        b'If you agree with the terms of this license, press "Agree" to install the software.  If you do not agree, press "Disagree".',  # noqa; E501
    ),
    "Deutsch": (
        b"Deutsch",
        b"Akzeptieren",
        b"Ablehnen",
        b"Drucken",
        b"Sichern...",
        b"Klicken Sie in \xd2Akzeptieren\xd3, wenn Sie mit den Bestimmungen des Software-Lizenzvertrags einverstanden sind. Falls nicht, bitte \xd2Ablehnen\xd3 anklicken. Sie k\x9annen die Software nur installieren, wenn Sie \xd2Akzeptieren\xd3 angeklickt haben.",  # noqa; E501
    ),
    "Español": (
        b"Espa\x96ol",
        b"Aceptar",
        b"No aceptar",
        b"Imprimir",
        b"Guardar...",
        b'Si est\x87 de acuerdo con los t\x8erminos de esta licencia, pulse "Aceptar" para instalar el software. En el supuesto de que no est\x8e de acuerdo con los t\x8erminos de esta licencia, pulse "No aceptar."',  # noqa; E501
    ),
    "Français": (
        b"Fran\x8dais",
        b"Accepter",
        b"Refuser",
        b"Imprimer",
        b"Enregistrer...",
        b'Si vous acceptez les termes de la pr\x8esente licence, cliquez sur "Accepter" afin d\'installer le logiciel. Si vous n\'\x90tes pas d\'accord avec les termes de la licence, cliquez sur "Refuser".',  # noqa; E501
    ),
    "Italiano": (
        b"Italiano",
        b"Accetto",
        b"Rifiuto",
        b"Stampa",
        b"Registra...",
        b'Se accetti le condizioni di questa licenza, fai clic su "Accetto" per installare il software. Altrimenti fai clic su "Rifiuto".',  # noqa; E501
    ),
    "Japanese": (
        b"Japanese",
        b"\x93\xaf\x88\xd3\x82\xb5\x82\xdc\x82\xb7",
        b"\x93\xaf\x88\xd3\x82\xb5\x82\xdc\x82\xb9\x82\xf1",
        b"\x88\xf3\x8d\xfc\x82\xb7\x82\xe9",
        b"\x95\xdb\x91\xb6...",
        b"\x96{\x83\\\x83t\x83g\x83E\x83G\x83A\x8eg\x97p\x8b\x96\x91\xf8\x8c_\x96\xf1\x82\xcc\x8f\xf0\x8c\x8f\x82\xc9\x93\xaf\x88\xd3\x82\xb3\x82\xea\x82\xe9\x8f\xea\x8d\x87\x82\xc9\x82\xcd\x81A\x83\\\x83t\x83g\x83E\x83G\x83A\x82\xf0\x83C\x83\x93\x83X\x83g\x81[\x83\x8b\x82\xb7\x82\xe9\x82\xbd\x82\xdf\x82\xc9\x81u\x93\xaf\x88\xd3\x82\xb5\x82\xdc\x82\xb7\x81v\x82\xf0\x89\x9f\x82\xb5\x82\xc4\x82\xad\x82\xbe\x82\xb3\x82\xa2\x81B\x81@\x93\xaf\x88\xd3\x82\xb3\x82\xea\x82\xc8\x82\xa2\x8f\xea\x8d\x87\x82\xc9\x82\xcd\x81A\x81u\x93\xaf\x88\xd3\x82\xb5\x82\xdc\x82\xb9\x82\xf1\x81v\x82\xf0\x89\x9f\x82\xb5\x82\xc4\x82\xad\x82\xbe\x82\xb3\x82\xa2\x81B",  # noqa; E501
    ),
    "Nederlands": (
        b"Nederlands",
        b"Ja",
        b"Nee",
        b"Print",
        b"Bewaar...",
        b"Indien u akkoord gaat met de voorwaarden van deze licentie, kunt u op 'Ja' klikken om de programmatuur te installeren. Indien u niet akkoord gaat, klikt u op 'Nee'.",  # noqa; E501
    ),
    "Svensk": (
        b"Svensk",
        b"Godk\x8anns",
        b"Avb\x9ajs",
        b"Skriv ut",
        b"Spara...",
        b'Om Du godk\x8anner licensvillkoren klicka p\x8c "Godk\x8anns" f\x9ar att installera programprodukten. Om Du inte godk\x8anner licensvillkoren, klicka p\x8c "Avb\x9ajs".',  # noqa; E501
    ),
    "Português": (
        b"Portugu\x90s",
        b"Concordar",
        b"Discordar",
        b"Imprimir",
        b"Salvar...",
        b'Se est\x87 de acordo com os termos desta licen\x8da, pressione "Concordar" para instalar o software. Se n\x8bo est\x87 de acordo, pressione "Discordar".',  # noqa; E501
    ),
    "Simplified Chinese": (
        b"Simplified Chinese",
        b"\xcd\xac\xd2\xe2",
        b"\xb2\xbb\xcd\xac\xd2\xe2",
        b"\xb4\xf2\xd3\xa1",
        b"\xb4\xe6\xb4\xa2\xa1\xad",
        b"\xc8\xe7\xb9\xfb\xc4\xfa\xcd\xac\xd2\xe2\xb1\xbe\xd0\xed\xbf\xc9\xd0\xad\xd2\xe9\xb5\xc4\xcc\xf5\xbf\xee\xa3\xac\xc7\xeb\xb0\xb4\xa1\xb0\xcd\xac\xd2\xe2\xa1\xb1\xc0\xb4\xb0\xb2\xd7\xb0\xb4\xcb\xc8\xed\xbc\xfe\xa1\xa3\xc8\xe7\xb9\xfb\xc4\xfa\xb2\xbb\xcd\xac\xd2\xe2\xa3\xac\xc7\xeb\xb0\xb4\xa1\xb0\xb2\xbb\xcd\xac\xd2\xe2\xa1\xb1\xa1\xa3",  # noqa; E501
    ),
    "Traditional Chinese": (
        b"Traditional Chinese",
        b"\xa6P\xb7N",
        b"\xa4\xa3\xa6P\xb7N",
        b"\xa6C\xa6L",
        b"\xc0x\xa6s\xa1K",
        b"\xa6p\xaaG\xb1z\xa6P\xb7N\xa5\xbb\xb3\\\xa5i\xc3\xd2\xb8\xcc\xaa\xba\xb1\xf8\xb4\xda\xa1A\xbd\xd0\xab\xf6\xa1\xa7\xa6P\xb7N\xa1\xa8\xa5H\xa6w\xb8\xcb\xb3n\xc5\xe9\xa1C\xa6p\xaaG\xa4\xa3\xa6P\xb7N\xa1A\xbd\xd0\xab\xf6\xa1\xa7\xa4\xa3\xa6P\xb7N\xa1\xa8\xa1C",  # noqa; E501
    ),
    "Dansk": (
        b"Dansk",
        b"Enig",
        b"Uenig",
        b"Udskriv",
        b"Arkiver...",
        b"Hvis du accepterer betingelserne i licensaftalen, skal du klikke p\x8c \xd2Enig\xd3 for at installere softwaren. Klik p\x8c \xd2Uenig\xd3 for at annullere installeringen.",  # noqa; E501
    ),
    "Suomi": (
        b"Suomi",
        b"Hyv\x8aksyn",
        b"En hyv\x8aksy",
        b"Tulosta",
        b"Tallenna\xc9",
        b"Hyv\x8aksy lisenssisopimuksen ehdot osoittamalla \xd5Hyv\x8aksy\xd5. Jos et hyv\x8aksy sopimuksen ehtoja, osoita \xd5En hyv\x8aksy\xd5.",  # noqa; E501
    ),
    "Korean": (
        b"Korean",
        b"\xb5\xbf\xc0\xc7",
        b"\xb5\xbf\xc0\xc7 \xbe\xc8\xc7\xd4",
        b"\xc7\xc1\xb8\xb0\xc6\xae",
        b"\xc0\xfa\xc0\xe5...",
        b'\xbb\xe7\xbf\xeb \xb0\xe8\xbe\xe0\xbc\xad\xc0\xc7 \xb3\xbb\xbf\xeb\xbf\xa1 \xb5\xbf\xc0\xc7\xc7\xcf\xb8\xe9, "\xb5\xbf\xc0\xc7" \xb4\xdc\xc3\xdf\xb8\xa6 \xb4\xad\xb7\xaf \xbc\xd2\xc7\xc1\xc6\xae\xbf\xfe\xbe\xee\xb8\xa6 \xbc\xb3\xc4\xa1\xc7\xcf\xbd\xca\xbd\xc3\xbf\xc0. \xb5\xbf\xc0\xc7\xc7\xcf\xc1\xf6 \xbe\xca\xb4\xc2\xb4\xd9\xb8\xe9, "\xb5\xbf\xc0\xc7 \xbe\xc8\xc7\xd4" \xb4\xdc\xc3\xdf\xb8\xa6 \xb4\xa9\xb8\xa3\xbd\xca\xbd\xc3\xbf\xc0.',  # noqa; E501
    ),
    "Norsk": (
        b"Norsk",
        b"Enig",
        b"Ikke enig",
        b"Skriv ut",
        b"Arkiver...",
        b'Hvis De er enig i bestemmelsene i denne lisensavtalen, klikker De p\x8c "Enig"-knappen for \x8c installere programvaren. Hvis De ikke er enig, klikker De p\x8c "Ikke enig".',  # noqa; E501
    ),
}

udifrezXMLtemplate = {
    "LPic": [
        {
            "Attributes": "0x0000",
            "Data": b"\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00",
            "ID": "5000",
            "Name": "",
        }
    ],
    "STR#": [
        {
            "Attributes": "0x0000",
            "Data": b'\x00\x06\x07English\x05Agree\x08Disagree\x05Print\x07Save...{If you agree with the terms of this license, press "Agree" to install the software.  If you do not agree, press "Disagree".',  # noqa; E501
            "ID": "5000",
            "Name": "English",
        },
    ],
    # ?? License text would be included in a block like this:
    # ?? 'TEXT': [
    # ??     {
    # ??         'Attributes': '0x0000',
    # ??         'Data': b'license text',
    # ??         'ID': '5000',
    # ??         'Name': 'English'
    # ??     }
    # ?? ],
    "TMPL": [
        {
            "Attributes": "0x0000",
            "Data": b"\x13Default Language IDDWRD\x05CountOCNT\x04****LSTC\x0bsys lang IDDWRD\x1elocal res ID (offset from 5000DWRD\x102-byte language?DWRD\x04****LSTE",  # noqa: E501
            "ID": "128",
            "Name": "LPic",
        }
    ],
    "styl": [
        {
            "Attributes": "0x0000",
            "Data": b"\x00\x03\x00\x00\x00\x00\x00\x0c\x00\t\x00\x14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'\x00\x0c\x00\t\x00\x14\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00*\x00\x0c\x00\t\x00\x14\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",  # noqa: E501
            "ID": "5000",
            "Name": "English",
        }
    ],
}


def maybe_encode(s, encoding="mac_roman"):
    if isinstance(s, bytes):
        return s
    return s.encode(encoding)


def build_license(license_info):
    """Add a license agreement to the specified disk image file, see
    https://developer.apple.com/forums/thread/668084."""
    # Copy the original template
    xml = dict(udifrezXMLtemplate)

    for language, license_data in license_info["licenses"].items():
        if language not in language_names_map:
            raise Exception(
                "Unknown language '"
                + language
                + "'. Valid languages are: "
                + ", ".join(sorted(language_names_map.keys()))
            )

        language_name = language_names_map[language]

        if os.path.isfile(license_data):
            mode = "rb" if license_data.endswith(".rtf") else "r"
            with open(license_data, mode=mode) as f:
                license_data = f.read()

        if type(license_data) == bytes and license_data.startswith(b"{\\rtf1"):
            licenseDataFormat = "RTF "

        else:
            licenseDataFormat = "TEXT"
            license_data = maybe_encode(license_data)

        if licenseDataFormat not in xml:
            xml[licenseDataFormat] = []

        xml[licenseDataFormat].append(
            {
                "Attributes": "0x0000",
                "Data": license_data,
                "ID": "5000",
                "Name": language_name,
            }
        )

        language_default_buttons = default_buttons.get(
            language_name, default_buttons["English"]
        )
        buttons = license_info.get("buttons", {}).get(
            language, language_default_buttons
        )

        assert len(buttons) == 6, "License buttons must have 6 entries."

        buttons = [maybe_encode(b) for b in buttons]
        xml["STR#"].append(
            {
                "Attributes": "0x0000",
                "Data": (
                    b"\x00\x06\x07"
                    + buttons[0]
                    + b"\x05"
                    + buttons[1]
                    + b"\x08"
                    + buttons[2]
                    + b"\x05"
                    + buttons[3]
                    + b"\x07"
                    + buttons[4]
                    + b"{"
                    + buttons[5]
                ),
                "ID": "5002",
                "Name": language_name,
            }
        )

    return xml
